<?php
defined('_JEXEC') or die;

class plgSystemEmailtools extends JPlugin
{  
    private $api_key;
    private $server_url;
    private $server_name;
    private $tracker_url = 'https://emailtools.ru/tracker/';
    private $payment_systems = [
        'jshopping' => ['YandexMoney20', 'YandexMoneySbbol'],
        'virtuemart' => [],
    ];
    private $log_filename = 'emt_log.txt';
    private $debug = true;
    private $plgVmOnAddToCartFilterCounter=0;



    public function __construct( & $subject, $config )
    {
        parent::__construct( $subject, $config );
        $this->api_key = $this->params->get('api_key', '');

        $this->payment_systems['virtuemart'] = explode(',', trim($this->params->get('virtuemart_payment_systems', '')));

        $this->server_name = $_SERVER ['HTTP_HOST'];
		$this->server_url = ($_SERVER['HTTPS'] == 'on') ? 'https://'.$this->server_name : 'http://'.$this->server_name;                
    }
        
    private function itsAllOk($withCookie = true)
    {
        $ok = true;
        if(empty($this->api_key)) $ok = false;
        if($withCookie)
        {
            if(!isset($_COOKIE['emt_uuuid']) || empty($_COOKIE['emt_uuuid'])) $ok = false;
        }
        return $ok;
    }

	private function log($operation, $response)
	{
		if (date ("Y-m-d", filemtime($this->log_filename)) !== date('Y-m-d')) 
		{
			unlink($this->log_filename);
		}
		$log  = "Date: ".date("F j, Y, g:i a").PHP_EOL.
        "Operation: ".$operation.PHP_EOL.
        "Response: ".$response.PHP_EOL.
        "-------------------------".PHP_EOL;
		file_put_contents($_SERVER["DOCUMENT_ROOT"].'/'.$this->log_filename, $log, FILE_APPEND);
	}

    private function getOrderProductIds($order, $shopType)
    {
        $result = [];
        switch($shopType)
        {
            case 'jshopping':
                $items = $order->getAllItems();
                foreach($items as $item)
                {
                    $result[] = $item->product_id;
                }
            break;
            case 'virtuemart':
                foreach($order->products as $item)
                {
                    $result[] = $item->virtuemart_product_id;
                }
            break;   
        }
        return implode(',', $result);
    }

    private function getName($order, $shopType)
    {
        $result = [];
        switch($shopType)
        {
            case 'jshopping':
                $result[] = $order->f_name;
                $result[] =$order->l_name;
            break;
            case 'virtuemart':
                $result[] = $order->BT['first_name'];
                $result[] = $order->BT['last_name'];
            break;   
        }
        return trim(implode(' ', $result));
    }

	private function operationSend($data)
	{
		$curl = curl_init();
		curl_setopt_array($curl, [
			CURLOPT_SSL_VERIFYPEER => false,
			CURLOPT_SSL_VERIFYHOST => false,
			CURLOPT_HTTPHEADER =>  [
				'Content-Type: application/x-www-form-urlencoded',
				'Accept: */*',
				'Origin: '.$this->server_url,
			],
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_URL => $this->tracker_url,
			CURLOPT_POST => true,
			CURLOPT_POSTFIELDS => http_build_query($data),
		]);
        $response = curl_exec($curl);
        curl_close($curl);
		if($this->debug)
		{
            $response = [
                'data' => $data,
                'response' => $response,
            ];
			$this->log($data['task'], print_r($response, true));
		}        
    }
    
    // joomla event
    public function onBeforeCompileHead()
    {
        if(!$this->itsAllOk(false)) return;
        $doc = JFactory::getDocument();
        JHtml::_('jquery.framework');
        $doc->addCustomTag('<script src="https://emailtools.ru/js/api/v1/tools.js" defer async></script>');
        $doc->addCustomTag('<script>window.EMT = window.EMT || {};EMT._client_id = '.$this->api_key.';</script>');
        $doc->addCustomTag('<script src="/plugins/system/emailtools/emailtools.js" defer async></script>');
    }

    // jshopping event
    public function onAfterCartAddOk($cart, $product_id)
    {
        if(!$this->itsAllOk()) return;
		$data = [
			'task' => 'addtocart',
			'client_id' => $this->api_key,
			'base_domain' => $this->server_name,
			'productid' => $product_id,
			'uuuid' => $_COOKIE['emt_uuuid'],
        ];
        $this->operationSend($data);
    }

    // jshopping event
    public function onAfterDisplayCheckoutFinish(&$text, &$order, &$pm_method)
    {
        if(!$this->itsAllOk()) return;

		if(in_array($pm_method->payment_code, $this->payment_systems['jshopping']))
		{
			$data = [
				'task' => 'onlinepay',
				'client_id' => $this->api_key,
				'base_domain' => $this->server_name,
				'uuuid' => $_COOKIE['emt_uuuid'],
				'email' => $order->email,
			];
			$this->operationSend($data);
        }

		$data = [
			'task' => 'sendOrder',
			'client_id' => $this->api_key,
			'base_domain' => $this->server_name,
			'uuuid' => $_COOKIE['emt_uuuid'],
			'name' => trim(implode(' ',[$order->f_name, $order->l_name])),
			'email' => $order->email,
			'orderid' => $order->order_id,
			'products' => $this->getOrderProductIds($order, 'jshopping'),
			'total' => (int) $order->order_total,
			'permission' =>  'subscribe' // unsubscribe
        ];
        $this->operationSend($data);
    }

    // virtuemart-3 event
    public function plgVmOnAddToCartFilter($product)
    {
        if(!$this->itsAllOk()) return;
		$data = [
			'task' => 'addtocart',
			'client_id' => $this->api_key,
			'base_domain' => $this->server_name,
			'productid' => $product->virtuemart_product_id,
			'uuuid' => $_COOKIE['emt_uuuid'],
        ];
        if($this->plgVmOnAddToCartFilterCounter==0)
        {
            $this->operationSend($data);
            $this->plgVmOnAddToCartFilterCounter++;
        }
    }

    private function getVmPaymentName($order)
    {
        $vmpayments = new SimpleXMLElement('<vmpayments>'.$order->cartData['paymentName'].'</vmpayments>');
        return $vmpayments->span[0];
    }

    // virtuemart-3 event
    public function plgVmConfirmedOrder($order)
    {
        if(!$this->itsAllOk(true)) return;

        if(!empty($this->payment_systems['virtuemart']) && in_array($this->getVmPaymentName($order), $this->payment_systems['virtuemart']))
		{
			$data = [
				'task' => 'onlinepay',
				'client_id' => $this->api_key,
				'base_domain' => $this->server_name,
				'uuuid' => $_COOKIE['emt_uuuid'],
				'email' => $order->BT['email'],
			];
			$this->operationSend($data);
        }

		$data = [
			'task' => 'sendOrder',
			'client_id' => $this->api_key,
			'base_domain' => $this->server_name,
			'uuuid' => $_COOKIE['emt_uuuid'],
			'name' => $this->getName($order, 'virtuemart'),
			'email' => $order->BT['email'],
			'orderid' => $order->virtuemart_order_id,
			'products' => $this->getOrderProductIds($order, 'virtuemart'),
			'total' => (int) $order->cartPrices['billTotal'],
			'permission' =>  'subscribe' // unsubscribe
        ];
        $this->operationSend($data); 
    }
    

}